import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.*;
import java.util.stream.Collectors;

public class E {

	public static void main(String[] args) throws IOException {
		sortFIPSGroups();
	}

	private static void sortFIPSGroups() throws IOException, IOException {
		
		// -- read FIPS-groups and Sort Commands from File
		String allContent = null;
		try (BufferedReader in = new BufferedReader(new InputStreamReader(System.in))) {
			allContent = in.readLine(); // <-- read whole line
		}

		if (allContent != null) {
			// -- Split Fips Groups from Commands
			String[] fipsGroupsAndCommandsString = allContent.split("-");

			List<String> fipsGruppenStringList = stringToList(fipsGroupsAndCommandsString[0], ",");
			// -- create FIPS-Gruppen Objects
			List<FipsGruppe> allFipsGruppen = new ArrayList<>();
			for (int i = 0; i < fipsGruppenStringList.size(); i++) {
				FipsGruppe fg = new FipsGruppe(fipsGruppenStringList.get(i), i);
				allFipsGruppen.add(fg);
			}

			// -- Parse SortAction Commands and FIPS-Gropus that need Sorting
			List<EnumSortAction> sortActionsList = new ArrayList<>();
			List<List<String>> fipsGruppenToBeSortedList = new ArrayList<List<String>>();
			String[] sortActionsAndFGroupsToBeSorted = fipsGroupsAndCommandsString[1].split(";");
			for (String sortActionAndFGroupToBeSorted : sortActionsAndFGroupsToBeSorted) {
				sortActionsList.add(EnumSortAction.valueOf(sortActionAndFGroupToBeSorted.split(":")[0]));
				fipsGruppenToBeSortedList.add(stringToList(sortActionAndFGroupToBeSorted.split(":")[1], ","));
			}

			long start = System.currentTimeMillis();
			
			// -- Sort FIPS-Groups
			for (int i = 0; i < sortActionsList.size(); i++) {
				allFipsGruppen = FipsGruppenSorter.sortFipsGruppen(sortActionsList.get(i),
						fipsGruppenToBeSortedList.get(i), allFipsGruppen);
			}

			long end = System.currentTimeMillis();
//			System.out.println("Sort time in millis: " + (end - start));
			
			// -- write sorted FIPS-groups to File
			List<String> stringListOut = allFipsGruppen.stream().map(fg -> fg.getFipsgruppe())
					.collect(Collectors.toList());

			System.out.println(String.join(",", stringListOut));
//			try (PrintWriter out = new PrintWriter("f.ans")) {
//				String content = listToString(stringListOut, ",");
//				out.println(content);
//			}
			
//			System.out.println("After sort: " + allFipsGruppen);
		}

	}

	public static String listToString(final Collection<String> stringList, String delimiter) {
		if (stringList == null) {
			return null;
		}
		
		StringBuilder sb = new StringBuilder();
		for (String s : stringList) {
			sb.append(s).append(delimiter);
		}
		
		String s = sb.toString();
		if (s.endsWith(delimiter)) {
			s = s.substring(0, s.length() - delimiter.length());
		}
		
		return s;
	}

	public static List<String> stringToList(String s, String delimiter) {
		if (s == null || s.length() == 0) {
			return new ArrayList<>();
		}
		
		String[] strings = s.split(delimiter);
		
		return Arrays.asList(strings);
	}

}

enum EnumSortAction {
    ONE_LEVEL_UP, ONE_LEVEL_DOWN, TO_HIGHEST_LEVEL, TO_LOWEST_LEVEL;
}

class FipsGruppe implements Comparable<FipsGruppe> {

    private String fipsgruppe;
    private int sort;
    
    public FipsGruppe(String fipsgruppe, int sort) {
        this.fipsgruppe = fipsgruppe;
        this.sort = sort;
    }
    
    public String getFipsgruppe() {
        return fipsgruppe;
    }
    
    public void setFipsgruppe(String fipsgruppe) {
        this.fipsgruppe = fipsgruppe;
    }
    
    public int getSort() {
        return sort;
    }
    
    public void setSort(int sort) {
        this.sort = sort;
    }
    
    @Override
    public String toString() {
        return this.fipsgruppe;
    }

    @Override
    public int compareTo(FipsGruppe o) {
        return Integer.compare(this.sort, o.sort);
    }
    
}

class FipsGruppenSortController {

    public static int setSortOneLevelUp(int minimumSort, int sort, Set<Integer> existingSortsCache) {
        if (sort == minimumSort) {
            if (existingSortsCache.contains(sort)) {
                return setSortOneLevelUp(minimumSort, sort + 1, existingSortsCache);
            } else {
                existingSortsCache.add(sort);
            }
        } else {
            int newSort = sort - 1;
            if (existingSortsCache.contains(newSort)) {
                return setSortOneLevelUp(minimumSort, sort + 1, existingSortsCache);
            } else {
                existingSortsCache.add(newSort);
                sort = newSort;
            }
        }

        return sort;
    }

    public static int setSortOneLevelDown(int maximumSort, int sort, Set<Integer> existingSortsCache) {
        if (sort == maximumSort) {
            if (existingSortsCache.contains(sort)) {
                return setSortOneLevelDown(maximumSort, sort - 1, existingSortsCache);
            } else {
                existingSortsCache.add(sort);
            }
        } else {
            int newSort = sort + 1;
            if (existingSortsCache.contains(newSort)) {
                return setSortOneLevelDown(maximumSort, sort - 1, existingSortsCache);
            } else {
                existingSortsCache.add(newSort);
                sort = newSort;
            }
        }

        return sort;
    }

}

class FipsGruppenSorter {

	public static List<FipsGruppe> sortFipsGruppen(EnumSortAction sortAction, List<String> fipsGruppenToBeSortedStrings,
            List<FipsGruppe> allFipsGruppen) {

        // -- Remove duplicate FipsGruppen
        Set<String> filteredFipsGruppenString = new HashSet<>(fipsGruppenToBeSortedStrings);

        // -- Get Sort Elements
        List<FipsGruppe> fipsGruppenToBeSorted = allFipsGruppen.stream().
                filter(fg -> filteredFipsGruppenString.contains(fg.getFipsgruppe())).collect(Collectors.toList());
        
        // -- Get Elements which don't nee Sort
        List<FipsGruppe> remainingFipsGruppen = allFipsGruppen.stream()
                .filter(fg -> !filteredFipsGruppenString.contains(fg.getFipsgruppe())).collect(Collectors.toList());

        // -- Get minimum and maximum Sort numbers from all existing FIPS-Gruppen
        int minimumSort = allFipsGruppen.stream().map(fg -> fg.getSort()).min(Comparator.comparing(Integer::valueOf))
                .get();
        int maximumSort = allFipsGruppen.stream().map(fg -> fg.getSort()).max(Comparator.comparing(Integer::valueOf))
                .get();

        // -- Select Sort Action
        switch (sortAction) {
        case ONE_LEVEL_UP:
            setFipsGruppenOneLevelUp(minimumSort, fipsGruppenToBeSorted, remainingFipsGruppen);
            break;
        case ONE_LEVEL_DOWN:
            setFipsGruppenOneLevelDown(maximumSort, fipsGruppenToBeSorted, remainingFipsGruppen);
            break;
        case TO_HIGHEST_LEVEL:
            setFipsGruppenToHighestLevel(minimumSort, fipsGruppenToBeSorted, remainingFipsGruppen);
            break;
        case TO_LOWEST_LEVEL:
            setFipsGruppenToLowestLevel(maximumSort, fipsGruppenToBeSorted, remainingFipsGruppen);
            break;
        }
        
        List<FipsGruppe> sortedFipsGruppen = new ArrayList<>();
        sortedFipsGruppen.addAll(fipsGruppenToBeSorted);
        sortedFipsGruppen.addAll(remainingFipsGruppen);
        Collections.sort(sortedFipsGruppen);
        
        return sortedFipsGruppen;
    }

    private static void setFipsGruppenOneLevelUp(int minimumSort, List<FipsGruppe> fipsGruppenToBeSorted,
            List<FipsGruppe> remainingFipsGruppen) {

        Set<Integer> existingSortsCache = new HashSet<>();
        // -- increase One Level
        fipsGruppenToBeSorted.forEach(fg -> fg
                .setSort(FipsGruppenSortController.setSortOneLevelUp(minimumSort, fg.getSort(), existingSortsCache)));
        remainingFipsGruppen.forEach(fg -> fg
                .setSort(FipsGruppenSortController.setSortOneLevelUp(minimumSort, fg.getSort(), existingSortsCache)));
    }

    private static void setFipsGruppenOneLevelDown(int maximumSort, List<FipsGruppe> fipsGruppenToBeSorted,
            List<FipsGruppe> remainingFipsGruppen) {
        Set<Integer> existingSortsCache = new HashSet<>();
        // -- decrease One Level
        Collections.reverse(fipsGruppenToBeSorted);
        fipsGruppenToBeSorted.forEach(fg -> fg.setSort(
                FipsGruppenSortController.setSortOneLevelDown(maximumSort, fg.getSort(), existingSortsCache)));
        Collections.reverse(remainingFipsGruppen);
        remainingFipsGruppen.forEach(fg -> fg.setSort(
                FipsGruppenSortController.setSortOneLevelDown(maximumSort, fg.getSort(), existingSortsCache)));
    }

    private static void setFipsGruppenToHighestLevel(int minimumSort, List<FipsGruppe> fipsGruppenToBeSorted,
            List<FipsGruppe> remainingFipsGruppen) {
        Set<Integer> existingSortsCache = new HashSet<>();
        // -- increase to Hihgest Level
        fipsGruppenToBeSorted.forEach(fg -> fg
                .setSort(FipsGruppenSortController.setSortOneLevelUp(minimumSort, fg.getSort(), existingSortsCache)));
        // -- get max Sort number from existing FipsGruppen
        if (!remainingFipsGruppen.isEmpty()) {
            int maximumSort = existingSortsCache.stream().max(Comparator.comparing(Integer::valueOf)).get();
            for (FipsGruppe fipsGruppe : remainingFipsGruppen) {
                fipsGruppe.setSort(++maximumSort);
            }
        }
    }

    private static void setFipsGruppenToLowestLevel(int maximumSort, List<FipsGruppe> fipsGruppenToBeSorted,
            List<FipsGruppe> remainingFipsGruppen) {
        Set<Integer> existingSortsCache = new HashSet<>();
        // -- decrease to Lowest Level
        Collections.reverse(fipsGruppenToBeSorted);
        fipsGruppenToBeSorted.forEach(fg -> fg.setSort(
                FipsGruppenSortController.setSortOneLevelDown(maximumSort, fg.getSort(), existingSortsCache)));
        // -- get min Sort number from existing FipsGruppen
        if (!remainingFipsGruppen.isEmpty()) {
            int minimumSort = existingSortsCache.stream().min(Comparator.comparing(Integer::valueOf)).get();
            Collections.reverse(remainingFipsGruppen);
            for (FipsGruppe fipsGruppe : remainingFipsGruppen) {
                fipsGruppe.setSort(--minimumSort);
            }
        }
    }
	
}
