#include <iostream>
#include <map>
#include <vector>
#include <math.h>
#include <time.h>
#include <algorithm>
#include <fstream>
#include <chrono>

using namespace std;

struct Pair{
    int x, y;
};

struct Inp{
    int i, type, a, b, answ;
    double time;
};

double distance(const Pair &a, const Pair &b){
    return sqrt( (a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y));
}

bool cmp_time(const Inp &a, const Inp &b){
    return a.time < b.time;
}

bool cmp_i(const Inp &a, const Inp &b){
    return a.i < b.i;
}

class Union{
private:
    int *parent;
    int *size;
    int n;
    int components;

public:
    Union(int n) : n(n) {
        parent = new int[n];
        size = new int[n];
        components = n;
        for(int i = 0; i < n; ++i){
            parent[i] = i;
            size[i] = 1;
        }
    }

    int findRoot(int x){
        while(x != parent[x]){
            parent[x] = parent[parent[x]];
            x = parent[x];
        }
        return x;
    }

    int areConnected(int x, int y){
        return findRoot(x) == findRoot(y);
    }

    void unite(int x, int y){
        int xr = findRoot(x);
        int yr = findRoot(y);
        if(xr == yr) return;

        if(size[xr] < size[yr]){
            parent[xr] = yr;
            size[yr] += size[xr];
        } else{
            parent[yr] = xr;
            size[xr] += size[yr];
        }
        components -= 1;
    }

    int getComponentsNumber() const {
        return components;
    }

};

int main(){
    //auto start = std::chrono::steady_clock::now();
    int n;

    vector<Pair> points;
    vector<Inp> inputs;
    map<double, vector<Pair> > dist;

    cin >> n;

    for(int i = 0; i < n; ++i){
        Pair tmp;
        cin >> tmp.x >> tmp.y;
        points.push_back(tmp);
    }

    int m;

    cin >> m;
    for(int i = 0; i < m; ++i){
        Inp tmp;
        tmp.i = i;
        cin >> tmp.type >> tmp.time;
        if(tmp.type){
            cin >> tmp.a >> tmp.b;
        }
        inputs.push_back(tmp);
    }


    for(int i = 0; i < n - 1; ++i){
        for(int j = i + 1; j < n; ++j){
            double d = distance(points[i], points[j])/2;
            Pair tmp;
            tmp.x = i;
            tmp.y = j;
            dist[d].push_back(tmp);
        }
    }

    Union u(n);
    sort(inputs.begin(), inputs.end(), cmp_time);
    int index = 0;
    for(auto &i : dist){
        double time = i.first;

        while(index < inputs.size() && time > inputs[index].time){
            if(inputs[index].type == 0){
                inputs[index].answ = u.getComponentsNumber();
            } else{
                inputs[index].answ = u.areConnected(inputs[index].a, inputs[index].b);
            }
            ++index;
        }

        vector<Pair> &pts = i.second;
        for(auto &p : pts){
            u.unite(p.x, p.y);
        }

    }

    while(index < inputs.size()){
        if(inputs[index].type == 0){
            inputs[index].answ = u.getComponentsNumber();
        } else{
            inputs[index].answ = u.areConnected(inputs[index].a, inputs[index].b);
        }
        ++index;
    }

    sort(inputs.begin(), inputs.end(), cmp_i);
    for(auto &i : inputs){
        cout << i.answ <<"\n";
    }
    //auto duration = std::chrono::duration_cast<std::chrono::milliseconds>
    //        (std::chrono::steady_clock::now() - start);
    //cout << duration.count();
    return 0;
}
